var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

import shallowEqual from 'shallowequal';
import { EventEmitter } from 'eventemitter3';

var ALLOWED_EVENT_TYPES = ['initialize', 'update', 'dispose'];

var checkAllowedEventType = function checkAllowedEventType(event) {
    if (!ALLOWED_EVENT_TYPES.some(function (e) {
        return event === e;
    })) {
        console.warn('[ObserverComponent] Event type "' + event + '" is not recognized, it will be ignored');
    }
};

export var ObserverComponent = function () {
    function ObserverComponent(store) {
        var _this = this;

        _classCallCheck(this, ObserverComponent);

        this.store = store;

        this._storeState = store.getState();
        this._stateProps = this.mapStateToProps(this._storeState);
        this._dispatchProps = this.mapDispatchToProps(this.store.dispatch);
        this._ee = new EventEmitter();

        this.props = _extends({}, this._stateProps, this._dispatchProps);

        this._unsubscribe = this.store.subscribe(function () {
            var nextStoreState = _this.store.getState();
            if (nextStoreState !== _this._storeState) {
                _this._storeState = nextStoreState;

                var nextStateProps = _this.mapStateToProps(_this._storeState);
                if (!shallowEqual(_this._stateProps, nextStateProps)) {
                    var prevProps = _this.props;
                    _this.props = _extends({}, nextStateProps, _this._dispatchProps);
                    _this.componentDidUpdate(prevProps);
                    _this._ee.emit('update', _this.props);
                }
            }
        });

        this.componentDidInitialize();
        this._ee.emit('initialize', this.props);
    }

    ObserverComponent.prototype.dispose = function dispose() {
        this.componentWillDispose();
        this._unsubscribe();
        this._ee.emit('dispose');
        this._ee.removeAllListeners();
    };

    ObserverComponent.prototype.on = function on(event, cb) {
        checkAllowedEventType(event);
        this._ee.on(event, cb);
        return this;
    };

    ObserverComponent.prototype.off = function off(event, cb) {
        checkAllowedEventType(event);
        this._ee.off(event, cb);
        return this;
    };

    ObserverComponent.prototype.mapStateToProps = function mapStateToProps(state) {};

    ObserverComponent.prototype.mapDispatchToProps = function mapDispatchToProps(dispatch) {};

    ObserverComponent.prototype.componentDidInitialize = function componentDidInitialize() {};

    ObserverComponent.prototype.componentDidUpdate = function componentDidUpdate(prevProps) {};

    ObserverComponent.prototype.componentWillDispose = function componentWillDispose() {};

    return ObserverComponent;
}();

export default ObserverComponent;

//# sourceMappingURL=index.esm.js.map